from argparse import ArgumentParser
from datetime import date

date_args = ArgumentParser(add_help=False)
date_args.add_argument(
    "--from",
    type=date.fromisoformat,
    help="date in ISO format",
    metavar="DATE",
    dest="from_date",
)
date_args.add_argument(
    "--to",
    type=date.fromisoformat,
    help="date in ISO format",
    metavar="DATE",
    dest="to_date",
)

json_arg = ArgumentParser(add_help=False)
json_arg.add_argument("-j", "--json", action="store_true", help="JSON output")

profile_arg = ArgumentParser(add_help=False)
profile_arg.add_argument("-P", "--profile", default="default")

debug_arg = ArgumentParser(add_help=False)
debug_arg.add_argument("--debug", action="store_true", help="Display debug info")

export_arg = ArgumentParser(add_help=False)
export_arg.add_argument("--export", choices=("html", "svg", "txt"))

team_arg = ArgumentParser(add_help=False)
team_arg.add_argument("-T", "--team", type=str, help="Run request as specific team")

config_arg = ArgumentParser(add_help=False)
config_arg.add_argument(
    "--config", type=str, help="Path to credentials file", metavar="PATH"
)

verbose_arg = ArgumentParser(add_help=False)
verbose_arg.add_argument(
    "-v", "--verbose", action="store_true", help="Display verbose info"
)



from contextlib import contextmanager
from importlib import metadata
from typing import Optional

from httpx import Client, Response, Timeout
from httpx_retries import Retry, RetryTransport

from .exceptions import SatoriRequestError

HOST = "https://api.satori.ci"
WS_HOST = "wss://api.satori.ci"
VERSION = metadata.version("satori-ci")


def raise_on_error(res: Response):
    if not res.is_success:
        try:
            res.read()
            message = res.json().get("detail", "Unknown error")
        except Exception:
            message = "Unknown error"

        raise SatoriRequestError(message, status_code=res.status_code)


timeout = Timeout(60.0, connect=10.0)
retry = Retry(total=3, backoff_factor=10)
transport = RetryTransport(retry=retry)
client = Client(
    base_url=HOST,
    follow_redirects=True,
    event_hooks={"response": [raise_on_error]},
    timeout=timeout,
    transport=transport,
)


def configure_client(
    token: str,
    team: Optional[str] = None,  # overwrite with --team/-T
    default_team: Optional[str] = None,  # set on profile
    timeout: Optional[int] = None,
    host: Optional[str] = None,
) -> None:
    client.headers.update(
        {
            "Authorization": f"Bearer {token}",
            "user-agent": f"satori-cli/{VERSION}",
        },
    )

    # If both team and default_team are provided, prefer the CLI argument
    selected_team = team or default_team
    if selected_team:
        client.headers["Satori-Team"] = selected_team

    if host:
        client.base_url = host

    if timeout:
        client.timeout = timeout


@contextmanager
def disable_error_raise():
    try:
        client.event_hooks["response"].remove(raise_on_error)
        yield client
    finally:
        client.event_hooks["response"].append(raise_on_error)



import re
import sys
from datetime import datetime
from importlib import metadata
from os import environ

import httpx
from packaging import version

from ..api import configure_client
from ..exceptions import SatoriRequestError
from ..utils import load_config
from .commands.config import ConfigCommand
from .commands.playbooks import PlaybooksCommand
from .commands.root import RootCommand
from .commands.shards import ShardsCommand
from .utils import configure_console_width, console, error_console, log, logging

VERSION = metadata.version("satori-ci")


def check_for_update():
    """Verify the current version and the latest version"""

    try:
        response = httpx.get("https://pypi.org/pypi/satori-ci/json")
        response.raise_for_status()
    except Exception:
        log.warning("Unable to get latest version.")
        return

    latest = response.json()["info"]["version"]

    if version.parse(latest) > version.parse(VERSION):
        log.warning("Newer version available v%s, update with: satori update", latest)


def main():
    timestamp = (datetime.now()).strftime("%Y-%m-%d %H:%M:%S")
    error_console.print(
        f"[dim]Satori CI {VERSION} - Automated Testing - Started on [b bright_magenta]{timestamp}"
    )
    root = RootCommand()
    args = root.parse_args()

    # Set debug/verbose mode
    if args["debug"]:
        log.setLevel(logging.DEBUG)
    elif args["verbose"]:
        log.setLevel(logging.WARNING)

    if environ.get("SATORI_CLI_NO_UPDATE_CHECK") != "1":
        check_for_update()

    if len(sys.argv) > 1 and sys.argv[1] == "update":
        try:
            exit_code = root.run(args)
            sys.exit(exit_code)
        except Exception as e:
            error_console.print(f"Error during update: {e}")
            sys.exit(1)

    try:
        full_config = load_config(args.get("config"))
        config = full_config.get(args["profile"])

        # Configure console width if specified in profile config
        if config and "width" in config:
            configure_console_width(config["width"])
    except Exception as e:
        if not isinstance(args["func"], ConfigCommand) and not isinstance(
            args["func"], ShardsCommand
        ):
            error_console.print(
                f"[error]ERROR:[/] Your .satori_credentials.yml file is corrupted or not found."
            )
            sys.exit(1)
        else:
            config = None

    if config:
        try:
            # Filter out width before passing to configure_client
            client_config = {k: v for k, v in config.items() if k != "width"}
            configure_client(**client_config)
        except Exception as e:
            error_console.print(
                f"[error]ERROR:[/] Cannot find your profile in your .satori_credentials.yml file. Is it corrupted?"
            )
            sys.exit(1)
    elif (
        not isinstance(args["func"], ConfigCommand)
        and not (
            # allow playbooks --public
            args["public"] and isinstance(args["func"], PlaybooksCommand)
        )
        and not isinstance(args["func"], ShardsCommand)
    ):
        # Allow config cmd only if profile not found
        error_console.print(f"[error]ERROR:[/] Profile {args['profile']} not found.")

        if config := load_config(args.get("config")):
            error_console.print("These are the profiles available:")

            for key in config:
                error_console.print(key)
        else:
            error_console.print(
                "\nGet your token at https://satori.ci/user-settings and configure it with:\n[b]satori config token TOKEN[/b]"
            )
        sys.exit(1)

    if config:
        config["team"] = args["team"]
        try:
            # Filter out width before passing to configure_client
            client_config = {k: v for k, v in config.items() if k != "width"}
            configure_client(**client_config)
        except Exception as e:
            error_console.print(
                f"[error]ERROR:[/] Cannot find your profile in your .satori_credentials.yml file. Is it corrupted?"
            )
            sys.exit(1)

    try:
        exit_code = root.run(args)
        export = args["export"]
        if export:
            if export == "html":
                content = console.export_html()
            elif export == "svg":
                content = console.export_svg(title="satori-cli")
                # remove non xml utf-8 chars
                regex = (
                    r"[^\x09\x0A\x0D\x20-\xFF\x85\xA0-\uD7FF\uE000-\uFDCF\uFDE0-\uFFFD]"
                )
                content = re.sub(regex, "", content, flags=re.MULTILINE)
            else:
                content = console.export_text()
            with open(f"output.{export}", "w") as f:
                f.write(content)
        sys.exit(exit_code)
    except SatoriRequestError as e:
        error_console.print(f"ERROR: {e}")
        error_console.print(f"Status code: {e.status_code}")
        sys.exit(1)
    except Exception as e:
        if args["debug"]:
            log.exception(e)
        else:
            error_console.print(e)
        sys.exit(1)
